'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (block) {
    switch (block.block) {
        case 'source':
            return source(block);
        case 'math':
            return math(block);
        default:
            (0, _logger.warn)('unknown block: ' + block.block);
            return block.value;
    }
};

var _prism = require('../prism');

var _katex = require('katex');

var _katex2 = _interopRequireDefault(_katex);

var _logger = require('../logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function source(block) {
    if (block.args.length > 0) {
        var language = block.args[0];
        if ((0, _prism.supports)(language)) {
            var highlighted = (0, _prism.highlight)(block.value, language);
            return '<pre class="preleganto-source"><code class="language-' + language + '">' + highlighted + '</code></pre>';
        } else {
            (0, _logger.warn)('unsupported source code language: ' + block.args[0]);
            return '<pre class="preleganto-source"><code>' + block.value + '</code></pre>';
        }
    } else {
        return '<pre class="preleganto-source"><code>' + block.value + '</code></pre>';
    }
}

function math(block) {
    var typeset = _katex2.default.renderToString(block.value, {
        displayMode: true
    });

    return '<div class="preleganto-display-math">' + typeset + '</div>';
}