'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _tipograph = require('tipograph');

var _tipograph2 = _interopRequireDefault(_tipograph);

var _parse = require('../syntax/parse');

var _parse2 = _interopRequireDefault(_parse);

var _template = require('./template');

var _template2 = _interopRequireDefault(_template);

var _theme = require('./theme');

var _theme2 = _interopRequireDefault(_theme);

var _macros = require('./macros');

var _macros2 = _interopRequireDefault(_macros);

var _blocks = require('./blocks');

var _blocks2 = _interopRequireDefault(_blocks);

var _logger = require('../logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Compiler = function () {
    function Compiler(rootpath) {
        _classCallCheck(this, Compiler);

        this._rootpath = rootpath;
    }

    _createClass(Compiler, [{
        key: 'compile',
        value: function compile(content) {
            var _this = this;

            var tree = (0, _parse2.default)(content);

            if (tree.name === 'Presentation') {
                var defaultMetadata = {
                    title: 'Preleganto presentation',
                    theme: 'default',
                    ratio: '16:10'
                };

                var metadata = Object.assign(defaultMetadata, getMetadata(tree.metadata));

                var template = new _template2.default();
                var theme = new _theme2.default(metadata.theme, this._rootpath);

                template.addCss(true, 'https://cdnjs.cloudflare.com/ajax/libs/normalize/7.0.0/normalize.min.css');
                template.addCss(true, 'https://cdnjs.cloudflare.com/ajax/libs/prism/1.6.0/themes/prism-okaidia.min.css');
                template.addCss(true, 'https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.7.1/katex.min.css');
                template.addCss(false, this._loadRequiredAsset('layout.css'));
                template.addCss(false, theme.renderStyle());

                template.addJs(false, this._loadRequiredAsset('control.js'));
                template.addJs(true, 'https://cdnjs.cloudflare.com/ajax/libs/prism/1.6.0/prism.min.js', {
                    'data-manual': true
                });
                template.addJs(true, 'https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.7.1/katex.min.js');

                if (metadata.customStyle) {
                    template.addCss(false, this._loadCustomAsset(metadata.customStyle));
                }

                if (metadata.customScript) {
                    template.addCss(false, this._loadCustomAsset(metadata.customScript));
                }

                if (metadata.title) {
                    template.setTitle(metadata.title);
                }

                template.setMetadata(metadata);

                template.addSlide(theme.renderOpening(metadata));
                tree.slides.forEach(function (slide) {
                    template.addSlide(theme.renderContent({
                        content: slide.value.map(function (element) {
                            return _compile(element, _this._rootpath);
                        }).join('\n')
                    }));
                });
                template.addSlide(theme.renderClosing(metadata));

                return template.toHtml();
            } else {
                return '';
            }
        }
    }, {
        key: '_loadRequiredAsset',
        value: function _loadRequiredAsset(asset) {
            var filepath = _path2.default.join(__dirname, '..', 'presentation', asset);
            try {
                return _fs2.default.readFileSync(filepath).toString();
            } catch (ex) {
                (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                return '';
            }
        }
    }, {
        key: '_loadCustomAsset',
        value: function _loadCustomAsset(asset) {
            var filepath = _path2.default.join(this._rootpath, asset);
            try {
                return _fs2.default.readFileSync(filepath).toString();
            } catch (ex) {
                (0, _logger.error)('load error of ' + filepath + ': you specified non-existent custom file');
                return '';
            }
        }
    }]);

    return Compiler;
}();

exports.default = Compiler;


function getMetadata(metadata) {
    return metadata.reduce(function (metadata, item) {
        var key = item.key.replace(/-(.)/g, function (match, letter) {
            return letter.toUpperCase();
        });
        metadata[key] = item.value;
        return metadata;
    }, {});
}

function typography(text) {
    return _tipograph2.default.Replace.all(text);
}

function _compile(node, rootpath) {
    switch (node.name) {
        case 'Heading':
            return '<h' + node.level + '>' + typography(node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('')) + '</h' + node.level + '>';
        case 'Paragraph':
            return '<p>' + typography(node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('')) + '</p>';
        case 'ListItem':
            return '<li>' + typography(node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('')) + '</li>';
        case 'OrderedList':
            return '<ol>' + node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('') + '</ol>';
        case 'UnorderedList':
            return '<ul>' + node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('') + '</ul>';
        case 'Text':
            return node.value;
        case 'TextStrong':
            return '<strong>' + node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('') + '</strong>';
        case 'TextEmph':
            return '<em>' + node.value.map(function (node) {
                return _compile(node, rootpath);
            }).join('') + '</em>';
        case 'TextMono':
            return '<code>' + _compile(node.value, rootpath) + '</code>';
        case 'TextMacro':
            return typography((0, _macros2.default)(node, rootpath));
        case 'SpecialBlock':
            return (0, _blocks2.default)(node);
        default:
            (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
            return '';
    }
}