'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (macro, rootpath) {
    switch (macro.macro) {
        case 'link':
            return link(macro, rootpath);
        case 'image':
            return image(macro, rootpath);
        case 'math':
            return math(macro);
        default:
            (0, _logger.warn)('unknown macro: ' + macro.macro);
            return macro.value;
    }
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _katex = require('katex');

var _katex2 = _interopRequireDefault(_katex);

var _logger = require('../logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isLocal(filepath) {
    var protocols = /^https?/;
    return !protocols.test(filepath);
}

function link(macro, rootpath) {
    var title = macro.value;
    if (macro.args.length > 0) {
        title = macro.args[0];
    }

    var url = macro.value;
    if (isLocal(url) && !_path2.default.isAbsolute(url)) {
        url = _path2.default.join(rootpath, url);
    }

    return '<a href="' + url + '">' + title + '</a>';
}

function image(macro, rootpath) {
    var alt = macro.value;
    var width = null;
    var height = null;

    if (macro.args.length > 0) {
        alt = macro.args[0];

        if (macro.args.length > 1) {
            width = macro.args[1];

            if (macro.args.length > 2) {
                height = macro.args[2];
            }
        }
    }

    var attributes = '';
    if (width !== null) {
        attributes += ' width="' + width + '"';
    }

    if (height !== null) {
        attributes += ' height="' + height + '"';
    }

    var source = macro.value;
    if (isLocal(source)) {
        var extension = _path2.default.extname(source).slice(1);
        source = _fs2.default.readFileSync(_path2.default.join(rootpath, source)).toString('base64');
        source = 'data:image/' + extension + ';base64,' + source;
    }

    return '<img src="' + source + '" alt="' + alt + '"' + attributes + ' />';
}

function math(macro) {
    return '<span class="preleganto-inline-math">' + _katex2.default.renderToString(macro.value) + '</span>';
}