'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _compiler = require('./build/compiler');

var _compiler2 = _interopRequireDefault(_compiler);

var _server = require('./server');

var _server2 = _interopRequireDefault(_server);

var _logger = require('./logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var commands = {
    build: 'Build a preleganto presentation into HTML file',
    serve: 'Serve a preleganto presentation on local server'
};

var options = {
    build: {
        input: {
            alias: 'i',
            required: true,
            description: 'A file with preleganto content'
        },
        output: {
            alias: 'o',
            default: 'slides.html',
            description: 'An output file'
        },
        watch: {
            alias: 'w',
            boolean: true,
            default: false,
            description: 'Watch the source file and rebuild presentation on change'
        }
    },
    serve: {
        input: {
            alias: 'i',
            required: true,
            description: 'A file with preleganto content'
        },
        port: {
            alias: 'p',
            default: 8080
        },
        watch: {
            alias: 'w',
            default: false,
            description: 'Watch the source file, rebuild presentation on change and reload all connected clients'
        }
    }
};

function build(input, output, options) {
    var compiler = new _compiler2.default(options.rootpath);

    var source = '';
    try {
        source = _fs2.default.readFileSync(input).toString();
    } catch (ex) {
        (0, _logger.error)('I can\'t find \'' + input + '\' :(');
    }

    var html = compiler.compile(source);

    try {
        _fs2.default.writeFileSync(output, html);
    } catch (ex) {
        (0, _logger.error)('I can\'t manage to write \'' + output + '\' :(');
    }
}

function watch(input, output, options) {
    var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : function () {};

    (0, _logger.log)('I am watching \'' + input + '\' for changes');
    (0, _logger.newline)();

    var lastTime = (0, _moment2.default)();
    _fs2.default.watch(input, function (eventType) {
        if (eventType === 'rename') {
            (0, _logger.log)('\'' + input + '\' was renamed or deleted wo I will stop watch it');
            process.exit();
        } else if (eventType === 'change') {
            var time = (0, _moment2.default)().format('H:mm:ss');

            // listener is triggered twice (empty file, write content)
            if (time !== lastTime) {
                lastTime = time;

                (0, _logger.log)('(' + time + ') \'' + input + '\' was changed so I will try to build it again');
                build(input, output, options);
                (0, _logger.log)('And I was successfull');
                callback();
            }
        }
    });
}

_yargs2.default.version().command('build', commands.build, options.build, function (argv) {
    var options = {
        rootpath: _path2.default.join(process.cwd(), _path2.default.dirname(argv.input))
    };

    (0, _logger.logo)();
    (0, _logger.newline)();
    (0, _logger.log)('I will try to build \'' + argv.input + '\'');

    build(argv.input, argv.output, options);

    (0, _logger.log)('I managed to build \'' + argv.input + '\' so I created \'' + argv.output + '\'');
    (0, _logger.newline)();

    if (argv.watch) {
        watch(argv.input, argv.output, options);
    }
}).command('serve', commands.serve, options.serve, function (argv) {
    var options = {
        rootpath: _path2.default.join(process.cwd(), _path2.default.dirname(argv.input))
    };

    var tempfile = _path2.default.join(options.rootpath, _path2.default.basename(argv.input, _path2.default.extname(argv.input))) + '.tmp';

    (0, _logger.logo)();
    (0, _logger.newline)();
    (0, _logger.log)('I will try to build \'' + argv.input + '\'');

    build(argv.input, tempfile, options);

    (0, _logger.log)('I managed to build \'' + argv.input + '\'');
    (0, _logger.log)('Now I am spawning local server which will serve the presentation');

    var server = new _server2.default({
        port: argv.port
    });

    server.run(tempfile);

    if (argv.watch) {
        watch(argv.input, tempfile, options, function () {
            return server.reloadClients();
        });
    }

    process.on('exit', function (code) {
        _fs2.default.unlinkSync(tempfile);

        if (code === 0) {
            (0, _logger.log)('Good job! I am shutting down the server now');
        }
    });

    process.on('SIGINT', function () {
        process.exit(0);
    });
}).help().strict(true).demandCommand().recommendCommands().argv;