'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

(function () {
    var utils = {
        select: function select(selector) {
            return document.querySelector(selector);
        },
        selectAll: function selectAll(selector) {
            return Array.from(document.querySelectorAll(selector));
        }
    };

    var Input = function () {
        function Input() {
            var _this = this;

            _classCallCheck(this, Input);

            this._handlers = [];

            // disable control keys
            window.addEventListener('keypress', function (ev) {
                if (_this._mapKeyboardEvent(ev) !== null) {
                    ev.preventDefault();
                }
            });

            // attach control keys' handlers
            // use keyup so it should get rid of accidental multiple slides moves
            window.addEventListener('keyup', function (ev) {
                var inputEvent = _this._mapKeyboardEvent(ev);

                if (inputEvent !== null) {
                    _this._emit(inputEvent);
                }
            });
        }

        _createClass(Input, [{
            key: 'on',
            value: function on(event, listener) {
                this._handlers.push({ event: event, listener: listener });
                return this;
            }
        }, {
            key: '_emit',
            value: function _emit(event) {
                this._handlers.forEach(function (handler) {
                    if (handler.event === event) {
                        handler.listener();
                    }
                });
            }
        }, {
            key: '_mapKeyboardEvent',
            value: function _mapKeyboardEvent(ev) {
                switch (ev.key) {
                    case 'ArrowLeft':
                    case 'ArrowUp':
                        return 'previous';
                    case 'ArrowRight':
                    case 'ArrowDown':
                    case ' ':
                    case 'Enter':
                        return 'next';
                    case 'r':
                    case 'R':
                        return 'ratio';
                    default:
                        return null;
                }
            }
        }]);

        return Input;
    }();

    var Preleganto = function () {
        function Preleganto() {
            _classCallCheck(this, Preleganto);

            this._initMetadata();
            this._initSlides();
            this._initInput();
            this._initSocket();
        }

        _createClass(Preleganto, [{
            key: 'goTo',
            value: function goTo(index) {
                var notify = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                if (index >= this._slides.length) {
                    this._current = this._slides.length - 1;
                } else if (index < 0) {
                    this._current = 0;
                } else {
                    this._current = index;
                }

                if (notify && this._socket !== null) {
                    var action = { name: 'goTo', index: index };
                    if (this._socket.readyState === WebSocket.OPEN) {
                        this._sendServerEvent(action);
                    } else {
                        this._actionQueue.push(action);
                    }
                }

                window.scrollTo(0, this._slides[this._current].offsetTop);
                window.location.hash = 'view-' + (this._current + 1);
                return this._current;
            }
        }, {
            key: 'previous',
            value: function previous() {
                return this.goTo(this._current - 1);
            }
        }, {
            key: 'next',
            value: function next() {
                return this.goTo(this._current + 1);
            }
        }, {
            key: 'switchRatio',
            value: function switchRatio() {
                var index = this._supportedRatios.indexOf(this._currentRatio);
                index = (index + 1) % this._supportedRatios.length;
                this._currentRatio = this._supportedRatios[index];
                this._setSlidesSize([window.innerWidth, window.innerHeight]);
            }
        }, {
            key: 'getCurrent',
            value: function getCurrent() {
                return this._current;
            }
        }, {
            key: 'getTotal',
            value: function getTotal() {
                return this._slides.length;
            }
        }, {
            key: 'getMetadata',
            value: function getMetadata() {
                return this._metadata;
            }
        }, {
            key: 'parseRatio',
            value: function parseRatio() {
                return [Number(this._currentRatio.split(':')[0]), Number(this._currentRatio.split(':')[1])];
            }
        }, {
            key: '_initMetadata',
            value: function _initMetadata() {
                var metadataScript = utils.select('script[type="text/preleganto-metadata"]');

                if (metadataScript) {
                    this._metadata = JSON.parse(metadataScript.textContent);
                } else {
                    console.warn('Cannot find Preleganto metadata. The presentation will run in limited mode.');
                    this._metadata = {
                        ratio: '16:10'
                    };
                }

                this._supportedRatios = ['16:10', '4:3', '16:9'];

                if (this._supportedRatios.indexOf(this._metadata.ratio) === -1) {
                    this._supportedRatios.push(this._metadata.ratio);
                }

                this._currentRatio = this._metadata.ratio;
            }
        }, {
            key: '_initSlides',
            value: function _initSlides() {
                var _this2 = this;

                this._slides = utils.selectAll('.preleganto-slide');

                if (/view-\d+/.test(window.location.hash)) {
                    this.goTo(Number(window.location.hash.match(/view-(\d+)/)[1]) - 1, false);
                } else {
                    this._current = 0;
                    window.location.hash = 'view-1';
                }

                // resize slides when window changed its size
                window.addEventListener('resize', function (ev) {
                    _this2._setSlidesSize([ev.target.innerWidth, ev.target.innerHeight]);
                    _this2.goTo(_this2._current, false);
                });

                this._setSlidesSize([window.innerWidth, window.innerHeight]);
            }
        }, {
            key: '_initInput',
            value: function _initInput() {
                var _this3 = this;

                this._input = new Input();

                this._input.on('previous', function () {
                    return _this3.previous();
                });
                this._input.on('next', function () {
                    return _this3.next();
                });
                this._input.on('ratio', function () {
                    return _this3.switchRatio();
                });
            }
        }, {
            key: '_initSocket',
            value: function _initSocket() {
                var _this4 = this;

                this._actionQueue = [];
                if (window.location.host === '') {
                    // presentation is served from filesystem
                    this._socket = null;
                } else {
                    // presentation is served from a server
                    this._socket = new WebSocket('ws://' + window.location.host);

                    this._socket.onopen = function () {
                        _this4._sendServerEvent({ name: 'sync' });
                        _this4._actionQueue.forEach(function (action) {
                            return _this4._sendServerEvent(action);
                        });
                    };

                    this._socket.onmessage = function (event) {
                        var action = JSON.parse(String(event.data));
                        switch (action.name) {
                            case 'goTo':
                                _this4.goTo(action.index, false);
                                break;
                            case 'sync':
                                _this4._sendServerEvent({ name: 'goTo', index: _this4._current });
                                break;
                            case 'reload':
                                // force browser to reload presentation from server
                                window.location.reload(true);
                                break;
                        }
                    };
                }
            }
        }, {
            key: '_setSlidesSize',
            value: function _setSlidesSize(dimensions) {
                var ratio = this.parseRatio();
                var width = Math.min(ratio[0] / ratio[1] * dimensions[1], dimensions[0]);
                var height = dimensions[1];

                this._slides.forEach(function (node) {
                    node.style.width = width + 'px';
                    node.style.height = height + 'px';
                });
            }
        }, {
            key: '_sendServerEvent',
            value: function _sendServerEvent(action) {
                if (this._socket !== null) {
                    this._socket.send(JSON.stringify(action));
                }
            }
        }]);

        return Preleganto;
    }();

    window.preleganto = new Preleganto();
})();