'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (content) {
    return parse((0, _tokenize2.default)(content), 0, 'ROOT')[0];
};

var _tokenize = require('./tokenize');

var _tokenize2 = _interopRequireDefault(_tokenize);

var _logger = require('../logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var node = {
    presentation: function presentation(metadata, slides) {
        return {
            name: 'Presentation',
            metadata: metadata,
            slides: slides
        };
    },
    metadata: function metadata(key, value) {
        return {
            name: 'Metadata',
            key: key,
            value: value
        };
    },
    slide: function slide(metadata, value) {
        return {
            name: 'Slide',
            metadata: metadata,
            value: value
        };
    },
    heading: function heading(level, value) {
        return {
            name: 'Heading',
            level: level,
            value: value
        };
    },
    paragraph: function paragraph(value) {
        return {
            name: 'Paragraph',
            value: value
        };
    },
    orderedList: function orderedList(value) {
        return {
            name: 'OrderedList',
            value: value
        };
    },
    unorderedList: function unorderedList(value) {
        return {
            name: 'UnorderedList',
            value: value
        };
    },
    listItem: function listItem(value) {
        return {
            name: 'ListItem',
            value: value
        };
    },
    text: function text(value) {
        return {
            name: 'Text',
            value: value
        };
    },
    textStrong: function textStrong(value) {
        return {
            name: 'TextStrong',
            value: value
        };
    },
    textEmph: function textEmph(value) {
        return {
            name: 'TextEmph',
            value: value
        };
    },
    textMono: function textMono(value) {
        return {
            name: 'TextMono',
            value: value
        };
    },
    textMacro: function textMacro(macro, value, args) {
        return {
            name: 'TextMacro',
            macro: macro,
            value: value,
            args: args
        };
    },
    specialBlock: function specialBlock(block, value, args) {
        return {
            name: 'SpecialBlock',
            block: block,
            value: value,
            args: args
        };
    }
};

function parse(tokens, pointer, state) {
    var level = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;

    if (state === 'ROOT') {
        var _metadata = [];
        while (pointer < tokens.length && tokens[pointer].name === 'Metadata') {
            _metadata.push(node.metadata(tokens[pointer].key, tokens[pointer].value));
            pointer++;
        }

        var _slides = [];
        if (pointer < tokens.length) {
            if (tokens[pointer].name === 'SlideSeparator') {
                if (pointer < tokens.length - 1) {
                    while (pointer < tokens.length) {
                        var slide = void 0;

                        var _parse = parse(tokens, pointer + 1, 'SLIDE');

                        var _parse2 = _slicedToArray(_parse, 2);

                        slide = _parse2[0];
                        pointer = _parse2[1];


                        if (slide.name === 'Slide') {
                            _slides.push(slide);
                        } else {
                            (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                        }
                    }
                }
            } else {
                (0, _logger.error)('syntax error: a slide separator must follow presentation metadata');
            }
        }

        return [node.presentation(_metadata, _slides), pointer];
    } else if (state === 'SLIDE') {
        var _metadata2 = [];
        var _value = [];
        var parsed = void 0;

        var token = tokens[pointer];
        while (pointer < tokens.length && token.name !== 'SlideSeparator') {
            if (token.name === 'Metadata') {
                _metadata2.push(node.metadata(token.key, token.value));
            } else if (token.name === 'Heading') {
                _value.push(node.heading(token.level, parseInline(token.value)));
            } else if (token.name === 'Paragraph') {
                _value.push(node.paragraph(parseInline(token.value)));
            } else if (token.name === 'OrderedListItem') {
                if (token.level === 1) {
                    var _parse3 = parse(tokens, pointer, 'ORDERED_LIST', 1);

                    var _parse4 = _slicedToArray(_parse3, 2);

                    parsed = _parse4[0];
                    pointer = _parse4[1];


                    if (parsed.name === 'OrderedList') {
                        _value.push(parsed);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else {
                    (0, _logger.error)('syntax error: list must start at the first level');
                }
            } else if (token.name === 'UnorderedListItem') {
                if (token.level === 1) {
                    var _parse5 = parse(tokens, pointer, 'UNORDERED_LIST', 1);

                    var _parse6 = _slicedToArray(_parse5, 2);

                    parsed = _parse6[0];
                    pointer = _parse6[1];


                    if (parsed.name === 'UnorderedList') {
                        _value.push(parsed);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else {
                    (0, _logger.error)('syntax error: list must start at the first level');
                }
            } else if (token.name === 'SpecialBlock') {
                _value.push(node.specialBlock(token.block, token.value, token.args));
            } else {
                (0, _logger.warn)('unimplemented token: ' + token.name);
            }

            token = tokens[++pointer];
        }

        return [node.slide(_metadata2, _value), pointer];
    } else if (state === 'ORDERED_LIST') {
        var items = [];
        var _token = tokens[pointer];
        while (pointer < tokens.length) {
            var item = void 0;

            if (_token.name === 'OrderedListItem') {
                if (_token.level > level) {
                    var _parse7 = parse(tokens, pointer, 'ORDERED_LIST', _token.level);

                    var _parse8 = _slicedToArray(_parse7, 2);

                    item = _parse8[0];
                    pointer = _parse8[1];


                    if (item.name === 'OrderedList') {
                        items.push(item);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else if (_token.level < level) {
                    pointer--;
                    break;
                } else {
                    items.push(node.listItem(parseInline(_token.value)));
                }
            } else if (_token.name === 'UnorderedListItem') {
                if (_token.level > level) {
                    var _parse9 = parse(tokens, pointer, 'UNORDERED_LIST', _token.level);

                    var _parse10 = _slicedToArray(_parse9, 2);

                    item = _parse10[0];
                    pointer = _parse10[1];


                    if (item.name === 'UnoderedList') {
                        items.push(item);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else {
                    pointer--;
                    break;
                }
            } else {
                pointer--;
                break;
            }

            _token = tokens[++pointer];
        }

        return [node.orderedList(items), pointer];
    } else if (state === 'UNORDERED_LIST') {
        var _items = [];
        var _token2 = tokens[pointer];
        while (pointer < tokens.length) {
            var _item = void 0;

            if (_token2.name === 'UnorderedListItem') {
                if (_token2.level > level) {
                    var _parse11 = parse(tokens, pointer, 'UNORDERED_LIST', _token2.level);

                    var _parse12 = _slicedToArray(_parse11, 2);

                    _item = _parse12[0];
                    pointer = _parse12[1];


                    if (_item.name === 'UnorderedList') {
                        _items.push(_item);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else if (_token2.level < level) {
                    pointer--;
                    break;
                } else {
                    _items.push(node.listItem(parseInline(_token2.value)));
                }
            } else if (_token2.name === 'OrderedListItem') {
                if (_token2.level > level) {
                    var _parse13 = parse(tokens, pointer, 'ORDERED_LIST', _token2.level);

                    var _parse14 = _slicedToArray(_parse13, 2);

                    _item = _parse14[0];
                    pointer = _parse14[1];


                    if (_item.name === 'OrderedList') {
                        _items.push(_item);
                    } else {
                        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
                    }
                } else {
                    pointer--;
                    break;
                }
            } else {
                pointer--;
                break;
            }

            _token2 = tokens[++pointer];
        }

        return [node.unorderedList(_items), pointer];
    } else {
        (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
        return [node.presentation([], []), pointer + 1];
    }
}

var REGEX_INLINE_MACRO = /^([A-Za-z]+):([^[\s][^[]+)\[([^\]]*)\]/;
var REGEX_INLINE_PLAIN = /^([^*_`]+)/;
var REGEX_INLINE_INNER_STRONG = /^\*\*(?!\s)([^(?:**)]+)\*\*/;
var REGEX_INLINE_INNER_EMPH = /^__(?!\s)([^(?:__)]+)__/;
var REGEX_INLINE_INNER_MONO = /^``(?!\s)([^(?:``)]+)``/;
var REGEX_INLINE_STRONG = /^\*(?!\s)([^*]+\S)\*/;
var REGEX_INLINE_EMPH = /^_(?!\s)([^_]+\S)_/;
var REGEX_INLINE_MONO = /^`(?!\s)([^`]+\S)`/;

function parseInline(value) {
    var output = [];

    while (value !== '') {
        if (REGEX_INLINE_MACRO.test(value)) {
            value = value.replace(REGEX_INLINE_MACRO, function (match, macro, value, args) {
                output.push(node.textMacro(macro, value, args.split(',').map(function (arg) {
                    return arg.trim();
                })));
                return '';
            });
        } else if (REGEX_INLINE_INNER_STRONG.test(value)) {
            value = value.replace(REGEX_INLINE_INNER_STRONG, function (match, value) {
                output.push(node.textStrong(parseInline(value)));
                return '';
            });
        } else if (REGEX_INLINE_INNER_EMPH.test(value)) {
            value = value.replace(REGEX_INLINE_INNER_EMPH, function (match, value) {
                output.push(node.textEmph(parseInline(value)));
                return '';
            });
        } else if (REGEX_INLINE_INNER_MONO.test(value)) {
            value = value.replace(REGEX_INLINE_INNER_MONO, function (match, value) {
                output.push(node.textMono(node.text(value)));
                return '';
            });
        } else if (REGEX_INLINE_STRONG.test(value)) {
            value = value.replace(REGEX_INLINE_STRONG, function (match, value) {
                output.push(node.textStrong(parseInline(value)));
                return '';
            });
        } else if (REGEX_INLINE_EMPH.test(value)) {
            value = value.replace(REGEX_INLINE_EMPH, function (match, value) {
                output.push(node.textEmph(parseInline(value)));
                return '';
            });
        } else if (REGEX_INLINE_MONO.test(value)) {
            value = value.replace(REGEX_INLINE_MONO, function (match, value) {
                output.push(node.textMono(node.text(value)));
                return '';
            });
        } else if (REGEX_INLINE_PLAIN.test(value)) {
            var regex = /[A-Za-z]+:[^[\s][^[]+\[[^\]]*\]/;

            // NOTE: there must be a better way how to handle macros
            if (regex.test(value)) {
                var macroLocation = value.indexOf(regex.exec(value)[0]);
                var plainLength = REGEX_INLINE_PLAIN.exec(value)[0].length;

                if (plainLength < macroLocation) {
                    value = value.replace(REGEX_INLINE_PLAIN, function (match, value) {
                        output.push(node.text(value));
                        return '';
                    });
                } else {
                    output.push(node.text(value.slice(0, macroLocation)));
                    value = value.slice(macroLocation);
                }
            } else {
                value = value.replace(REGEX_INLINE_PLAIN, function (match, value) {
                    output.push(node.text(value));
                    return '';
                });
            }
        } else {
            (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
        }
    }

    return output;
}